d3.select("body").append("div").attr("class", "tip").style("display", "none");

var columns = ['length', 'diameter', 'height', 'whole_weight', 'shucked_weight', 'viscera_weight', 'shell_weight', 'rings'];

const render = (data, svg, title) => {

  var corr = jz.arr.correlationMatrix(data, columns);
  var extent = d3.extent(corr.map(function (d) { return d.correlation; }));
  var grid = data2grid.grid(corr);
  var rows = columns.length;
  var margin = { top: 30, bottom: 30, left: 90, right: 0 };
  var dim = d3.min([window.innerWidth * 1.5, window.innerHeight * 1.5])
  var width = dim * 0.6, height = dim * 0.6;

  svg = svg
    .attr("width", dim)
    .attr("height", dim)
    .append("g")
    .attr("transform", "translate(" + margin.left + ", " + margin.top + ")");

  svg.append('text').text(title)
    .attr('class', 'title')
    .attr("text-anchor", "middle")
    .attr("transform", "translate(" + (width / 2) + ", " + -10 + ")"); // 調整標題位置

  var padding = .2; // set padding
  var x = d3.scaleBand()
    .range([0, width])
    .paddingInner(padding)
    .domain(d3.range(1, rows + 1));
  var y = d3.scaleBand()
    .range([0, height])
    .paddingInner(padding)
    .domain(d3.range(1, rows + 1));


  var x_axis = d3.axisBottom(x).tickFormat(function (d, i) { return columns[i]; });
  var y_axis = d3.axisLeft(y).tickFormat(function (d, i) { return columns[i]; });

  var axisXcreated = svg.append("g")
    .attr("class", "x axis")
    .call(x_axis)
    .attr("transform", "translate(0, " + height + ")");

  axisXcreated.selectAll('text')
    .style('text-anchor', 'end')
    .attr('dx', '-0.5em')
    .attr('dy', '0.5em')
    .attr('transform', 'rotate(-45)'); ;

  svg.append("g")
    .attr("class", "y axis")
    .call(y_axis);
  
  // create scaler for color
  var colorscale = chroma.scale(["red", "white", "steelblue"]).domain([-1, 0, extent[1]]);

  // draw the rectengles
  svg.selectAll("rect")
    .data(grid, function (d) { return d.column_a + d.column_b; })
    .enter().append("rect")
    .attr("x", function (d) { return x(d.column); })
    .attr("y", function (d) { return y(d.row); })
    .attr("width", x.bandwidth())
    .attr("height", y.bandwidth())
    .style("fill", function (d) { return colorscale(d.correlation); })
    .style("opacity", 10);

  svg.selectAll("rect")
    .on("mouseover", function (d) {
      d3.select(this).classed("selected", true);

      d3.select(".tip")
        .style("display", "block")
        .html(d.column_x + ", " + d.column_y + ": " + d.correlation.toFixed(2));

      var row_pos = y(d.row);
      var col_pos = x(d.column);
      var tip_pos = d3.select(".tip").node().getBoundingClientRect();
      var tip_width = tip_pos.width;
      var tip_height = tip_pos.height;
      var grid_pos = svg.node().getBoundingClientRect();
      var grid_left = grid_pos.left;
      var grid_top = grid_pos.top;
      var left = grid_left + col_pos + margin.left;
      var top = grid_top + row_pos + margin.top - tip_height - 5;

      d3.select(".tip")
        .style("left", left + "px")
        .style("top", top + "px");

      svg.select(".x.axis .tick:nth-of-type(" + d.column + ") text").classed("selected", true);
      svg.select(".y.axis .tick:nth-of-type(" + d.row + ") text").classed("selected", true);
      svg.select(".x.axis .tick:nth-of-type(" + d.column + ") line").classed("selected", true);
      svg.select(".y.axis .tick:nth-of-type(" + d.row + ") line").classed("selected", true);

    })
    .on("mouseout", function () {
      svg.selectAll("rect").classed("selected", false);
      d3.select(".tip").style("display", "none");
      svg.selectAll(".axis .tick text").classed("selected", false);
      svg.selectAll(".axis .tick line").classed("selected", false);
    });

};

const dropdown = d3.select("body").append("div").attr("class", "dropdown").style("text-align", "center"); // 新增下拉式選單容器
dropdown.append("h2").text("Select Gender:"); // 新增標題

const select = dropdown.append("select")
  .attr("id", "genderSelect")
  .style("margin", "10px")
  .on("change", function () {
    const selectedGender = this.value;
    d3.selectAll(".svg-container").style("display", "none");
    d3.select(`#${selectedGender}-svg`).style("display", "block");
  });

select.selectAll("option")
  .data(["Male", "Female", "Infant"])
  .enter().append("option")
  .attr("value", d => d)
  .text(d => d);

d3.text("abalone.csv", function (text) {
  var data = d3.csvParseRows(text, (d, i) => {
    return {
      sex: d[0],
      length: +d[1],
      diameter: +d[2],
      height: +d[3],
      whole_weight: +d[4],
      shucked_weight: +d[5],
      viscera_weight: +d[6],
      shell_weight: +d[7],
      rings: +d[8]
    };
  });

  const svgM = d3.select("body").append("div").attr("class", "svg-container").attr("id", "Male-svg").style("display", "none")
    .append("svg");
  const svgF = d3.select("body").append("div").attr("class", "svg-container").attr("id", "Female-svg").style("display", "none")
    .append("svg");
  const svgI = d3.select("body").append("div").attr("class", "svg-container").attr("id", "Infant-svg").style("display", "none")
    .append("svg");

  render(data.filter(d => d.sex === 'M'), svgM, 'Male');
  render(data.filter(d => d.sex === 'F'), svgF, 'Female');
  render(data.filter(d => d.sex === '